import json
from pprint import pprint
import requests
import logging

class Profile:
    '''
    This is a constructor for the class Profile.
    :param idToken: The access token for the user.
    :return: None
    '''
    def __init__(self, idToken):
        self.idToken = idToken
        self.headers = {
            "access-key": self.idToken,
            "content-type": "application/json",
            "correlationid": "1649245883892::980870b94e2a6013b6859c251e0fa7ee",
            "origin": "https://app.senecalearning.com"
        }
        
    def getAvatarId(self, name: str):  
        '''
        Gets the avatar id of the user from the avatars.app.senecalearning.com API.
        :param name: The name of the avatar.
        :return: The avatar id of the user.
        '''
        url = "https://avatars.app.senecalearning.com/api/avatars"
        response = requests.request("GET", url, headers=self.headers)
        data = response.json()
        data.get('name')
        for i in data.get('avatars'):
            if i.get('name') == name:
                return i.get('avatarId')
        logging.error('Invalid AvatarName')
        return None
    
    def setAvatar(self, AvatarId):
        '''
        Sets the avatar of the user.
        :param AvatarId: The id of the avatar.
        :return: The name of the avatar.
        '''
        url = "https://avatars.app.senecalearning.com/api/avatars"
        payload = {"avatarId": AvatarId}
        response = requests.request("POST", url, json=payload, headers=self.headers)
        AvatarName = response.json().get('name')
        return AvatarName
    def generateNewDisplayName(self):
        '''
        Generates a new display name for the user.
        :return: The new display name.
        '''
        url = "https://user-info.app.senecalearning.com/api/user-info/me/update"
        payload = {"displayName": "!generate"}
        response = requests.request("POST", url, json=payload, headers=self.headers)
        NewDisplayName = response.json().get('displayName')
        return NewDisplayName


import json
from pprint import pprint
import requests


class Memories:
    '''
    This class is used to store the data from the memory app.
    :param idToken: The token used to access the data.
    :return: None
    '''
    def __init__(self, idToken):
        self.idToken = idToken
        url = "https://memory.app.senecalearning.com/api/locations"
        querystring = {"limit":"2000"}
        headers = {"correlationid": "1647102551794::327892c76c7fed9f1443ebe7fac9c258"}
        response = requests.request("GET", url, headers=headers, params=querystring)
        self.data = json.loads(response.text).get('items')

    def __ParseInfo(self, i):
        '''
        Parses the information from the json file and returns a list of the required information.
        :param i: The information from the json file.
        :return: A list of the required information.
        '''
        loc = []
        loc.append(i.get('countryName'))
        loc.append(i.get('name'))
        loc.append(i.get('id'))
        return loc

    def AllCountries(self):
        '''
        This function returns a list of all the countries in the dataset.
        :return: A list of all the countries in the dataset.
        '''
        Countries = []
        for i in self.data:
            Countries.append((self.__ParseInfo(i)))
        return Countries
    def SetMemory(self, courseId, sectionId, locationId, courseName ,sectionTitle):
        '''
        Sets a memory in the Seneca Learning Memory API.
        :param courseId: The course ID of the course.
        :param sectionId: The section ID of the section.
        :param locationId: The location ID of the location.
        :param courseName: The name of the course.
        :param sectionTitle: The title of the section.
        :return: None
        '''
        url = "https://memory.app.senecalearning.com/api/memories"

        payload = {
            "courseId": courseId,
            "courseName": courseName,
            "sectionId": sectionId,
            "sectionTitle": sectionTitle,
            "locationId": locationId,
            "timeEarned": "2022-03-21T17:55:31+00:00"
        }
        headers = {
            "correlationid": "1647885342893::8fb5230c62f1c5be3acec6582830e7b5",
            "access-key": self.idToken,
        }

        response = requests.request("POST", url, json=payload, headers=headers)
        if response.status_code == 201:
            print(response)
            print('Set!')
            pprint(response.json().get('courseName'))
        else:
            print('Error')
            pprint(response.json())

        

    def GetLocation(self, name:str):
        '''
        Gets the location of a location.
        :param name: The name of the location.
        :return: The latitude and longitude of the location.
        '''
        for i in self.data:
            if i.get('name') == name:
                print('Found Location for:')
                self.name = print(self.__ParseInfo(i))
                Coords = []
                Coords.append(i.get('latitude'))
                Coords.append(i.get('longitude'))

                return Coords





from datetime import datetime
from http.client import ResponseNotReady
from websocket import create_connection
import requests
from seneca.Modules.data import Create
import threading
from pprint import pprint
import json
import logging
class Sessions():
    '''
    This is a constructor for the Sessions class.
    :param SessionId: The sessionId of the user.
    :param User: The user object.
    '''
    def __init__(self,  SessionId:str, User:any): 
        self.SessionId = SessionId
        self.idToken = User.idToken
        self.userId = User.id
        
    def Start(self):
        '''
        Starts the session and starts a thread to keep track of the xp.
        :return: None
        '''
        self.ws = create_connection(f"wss://session-ws.app.senecalearning.com/?access-key={self.idToken}&sessionId={self.SessionId}")
        logging.info('Connected to websocket')
        data = '{"action":"start-session","data":{"userId":"'+  self.userId +'","sessionId":"' + self.SessionId + '","courseId":"2670ac10-1d69-11e8-bf76-f14a3ef7c0e6","sectionId":"eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a"}}'
        logging.info('Starting session')
        self.ws.send(data)
        logging.info("Session ready")
        def thread_function(self):
            '''
            This function is used to collect the data from the web socket.
            :param self: The self parameter is a reference to the class instance itself.
            :return: None
            '''
            
            total = 0
            while True:
                data = self.ws.recv()
                try:
                    data = json.loads(data)
                except:
                    break
                data = data.get('data')
                TotalXp = data.get('totalXp')
                total += 1
                time = datetime.now()
                current = time.strftime("%H:%M:%S")
                print(f"{total}. TotalXp: {TotalXp} Time: {current}")
        x = threading.Thread(target=thread_function, args=(self,))
        x.start()
        
    def End(self):
        '''
        Closes the connection to the websocket.
        '''
        self.ws.close()
        
        
        
    def SubmitData(self):
        '''
        Submits the data to the server.
        :return: The response from the server.
        '''
        url = "https://stats.app.senecalearning.com/api/stats/sessions"

        payload =  Create(self.SessionId)
        headers = {
            "authority": "stats.app.senecalearning.com",
            "pragma": "no-cache",
            "cache-control": "no-cache",
            "correlationid": "1648235477567::180297b651c5a6ff4607a9288d29cdf5",
            "user-region": "GB",
            "sec-ch-ua-mobile": "?0",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.82 Safari/537.36",
            "content-type": "application/json",
            "access-key": self.idToken,
            "x-amz-date": "20220325T191117Z",
            "accept": "*/*",
            "origin": "https://app.senecalearning.com",
            "sec-fetch-site": "same-site",
            "sec-fetch-mode": "cors",
            "sec-fetch-dest": "empty",
            "referer": "https://app.senecalearning.com/",
            "accept-language": "en-GB,en-US;q=0.9,en;q=0.8"
        }

        response = requests.request("POST", url, json=payload, headers=headers)

        if response.status_code == 200:
            return response
        else:
            return 'Error'






import requests
from seneca.Modules.errors import (
    CourseNotFound, InvalidUrl, NoPremiumSubscriptionFound)
from seneca.Modules.key import CORRELATOINID


class Course:
    '''
    This is a class for the Course.
    '''

    def __init__(self):
        pass

    def _parseUrl(self, url):
        '''
        Parses the url and returns the courseId and sectionId.
        :param url: The url to be parsed.
        :return: A list containing the courseId and sectionId.
        '''
        try:
            courseId = url.split('course/')[1].split('/section')[0]
            SectionId = url.split('section/')[1].split('/session')[0]
            return [courseId, SectionId]
        except:
            raise InvalidUrl(url)

    def _Parse(self, data):
        '''
        Parses the data from the API and returns a dictionary of the questions and answers.
        :param data: The data from the API.
        :return: A dictionary of the questions and answers.
        '''

        # Finds Modules Ids and Number of Modules
        NumberOfModules = 0
        ModuleIds = []
        for i in data.get("moduleIds"):
            NumberOfModules += 1
            ModuleIds.append(i)

        # Finds title
        title = data.get('title')

        # Output
        print(f'---{title}---')
        print(f'NumberOfQuestions:\n{NumberOfModules}\n')

        Modules = []
        contents = data.get('contents')
        Question = 0
        for i in contents:
            contentModules = i.get('contentModules')
            for i in contentModules:
                Modules.append(
                    {Question: [i.get('content'), i.get('moduleType')]})

                Question += 1

        def Calculate(Type, q):
            if Type == 'equation':
                a = [q.get('title')]
                a.append(q.get('blocks')[0].get('word'))
                return a
            elif Type == 'multiple-choice':
                a = [q.get('statement')]
                a.append(q.get('correctAnswer'))
                return a
            elif Type == 'toggles':
                a = [q.get('statement')]
                amount = q.get('toggles')
                Correct = []
                for i in amount:
                    Correct.append(i.get('correctToggle'))
                a.append(Correct)
                return a
            elif Type == 'wordfill':
                a = [Type]
                a.append(q.get('words')[-2].get('word'))
                return a
            elif Type == 'worked-example':
                a = [q.get('question')]
                steps = q.get('steps')
                Correct = []
                for i in steps:
                    try:
                        Correct.append(i.get('equation')[1].get('word'))
                    except:
                        continue
                a.append(Correct)
                return a

            elif Type == 'grid':
                a = [q.get('title')]
                Correct = []
                for i in q.get('definitions'):
                    try:
                        Correct.append(i.get('word')[1].get('word'))
                    except:
                        continue
                a.append(Correct)
                return a

            elif Type == 'exact-list':
                a = [q.get('statement')]
                Correct = []
                for i in q.get('values'):
                    Correct.append(i.get('value')[0].get('word'))
                a.append(Correct)
                return a

            elif Type == 'multiSelect':
                a = [q.get('question')]
                Correct = []
                for i in q.get('options'):
                    if i.get('correct') == True:
                        Correct.append(i.get('text'))
                a.append(Correct)
                return a
            elif Type == 'image':
                a = [Type]
                a.append(q)
                return a

        Answers = {}
        for i in range(len(Modules)):
            q = Modules[i].get(i)
            Type = q[1]
            Content = q[0]
            # continue
            Result = Calculate(Type, Content)
            if Result:
                Answers[i] = Result

        return Answers

    def QueryCourses(self, number: int, filters:dict=None):
        '''
        This function queries the seneca app api for a number of courses.
        :param number: The number of courses to be queried.
        :return: A list of courses.
        '''

        url = "https://course.app.senecalearning.com/api/courses/queryCourses"

        payload = {
            "aggregations": [
                "bdf3e1a7-ef2e-46a7-8e70-52d3370da3da",
                "be426768-2fd2-4231-9c25-22354a89c1b2",
                "97425b6f-d99e-4066-8ac5-dfb2c7c1519a",
                "8c71d055-ff40-4536-8629-5073c618ea53",
                "29304df7-aad4-4b2f-ac43-2fb169c04ebd",
                "3a8d961c-3623-422e-8d7e-8606191a7bca",
                "3df4913a-5a19-4777-9fdf-1c1d74f40491"
            ],

            "query": "",

            "size": number
        }
        if filters:
            payload['filters'] = filters
        headers = {
            "correlationid": "1647887386631::02e946994d35a04376af966b404386fc"}

        response = requests.request("POST", url, json=payload, headers=headers)
        courses = response.json()
        courses = courses.get('results').get('hits')
        return courses

    def getAnswers(self, url: str):
        '''
        This function gets the answers from the given url.
        :param url: The url of the course.
        :return: A list of answers.
        '''
        Course = self.getCourseInfo(url)
        Answers = self._Parse(Course)
        return Answers

    def getCourseInfo(self, url: str, user=None):
        '''
        This function parses the url and extracts the course id and section id.
        :param url: The url of the course.
        :return: A list containing the course id and section id.
        '''

        Template = [
            'https://course.app.senecalearning.com/api/courses/', '/signed-url']
        Ids = self._parseUrl(url)
        Template.insert(1, Ids[0])
        self.url = ''.join(Template)
        querystring = {"sectionId": Ids[1]}
        headers = {
            "authority": "course.app.senecalearning.com",
            "correlationid": CORRELATOINID
        }
        if user:
            headers["access-key"] = user.idToken
        response = requests.request(
            "GET", self.url, headers=headers, params=querystring)
        if response.json().get('reason'):
            raise NoPremiumSubscriptionFound()

        if response.status_code != 200:
            raise CourseNotFound()
        self.url = response.json().get('url')
        response = requests.request("GET", self.url, headers=headers)
        Course = response.json()
        return Course




"""File for defining errors"""


class SenecaError(Exception):
    """Base class for Seneca errors."""

class CourseNotFound(SenecaError):
    pass


class InvalidUrl(SenecaError):
    def __init__(self, url):
        super().__init__(str(url))

class InvalidCredentials(SenecaError):
    pass


class ServerError(SenecaError):
    pass
class NoPremiumSubscriptionFound(SenecaError):
    pass


from ast import Subscript
import requests
from seneca.Modules.key import  APIKEY, CORRELATOINID
class Info:
    '''
    This is a constructor for the class Info.
    :param idToken: The access key for the API.
    '''
    def __init__(self, idToken): 
        self.idToken = idToken
        self.headers =  {
            'access-key' : self.idToken,
            "origin": "https://app.senecalearning.com"
        }
    def classInfo(self):
        pass
    def getAccountInfo(self):
        '''
        Sends a GET request to the server for the user information.
        :return: The user Json object.
        '''
        """Sends a GET request to the server for the user information

        Returns:
            json: The user Json object
        """        
        url = "https://www.googleapis.com/identitytoolkit/v3/relyingparty/getAccountInfo"
        querystring = {"key": APIKEY} #We need this key otherwise we cant access the account information
        payload = {"idToken": self.idToken} #This token specifies the account we are getting the information from

        #Sending Request to get the user information
        response = requests.request("POST", url, json=payload, headers=self.headers, params=querystring)


        #Parsing Response
        AccountInfo = response.json().get('users')[0]
        self.displayName = AccountInfo.get('displayName')
        self.email = AccountInfo.get('email')
        self.lastRefreshAt = AccountInfo.get('lastRefreshAt')
        self.passwordHash = AccountInfo.get('UkVEQUNURUQ=')
        self.photoUrl = AccountInfo.get('photoUrl')
        self.createdAt = AccountInfo.get('createdAt')
        return AccountInfo
    
    def _me(self, url):
        '''
        This function is used to make a GET request to the API.
        :param url: The URL of the API.
        :return: The response of the GET request.
        '''
        self.headers['correlationid'] = '1647206128958::50f6180dec98105c2f1317155089ce1d'
        response = requests.request("GET", url, headers=self.headers)
        data = response.json()
        return data
    
    def getSchoolData(self):
        '''
        This function gets the school data from the seneca app.
        :return: A dictionary of the school data.
        '''
        url = 'https://schools.app.senecalearning.com/api/school-info/me'
        SchoolInfo =  self._me(url)
        return SchoolInfo
    
    def getUserData(self):
        '''
        This function gets the user info from the API.
        :return: A dictionary of the user info.
        '''
        url = 'https://user-info.app.senecalearning.com/api/user-info/me'
        UserInfo = self._me(url)
        return UserInfo

    
    
    def getSubscriptionData(self):
        '''
        This function gets the subscription data from the subscription API.
        :return: A dictionary of the subscription data.
        '''
        url = 'https://subscriptions.app.senecalearning.com/api/subscriptions/me'
        SubscriptionData =  self._me(url)
        return SubscriptionData
    def getKnowledgeScore(self):
        '''
        This function returns the KnowledgeScore of the user.
        :return: KnowledgeScore of the user.
        '''
        url = 'https://stats.app.senecalearning.com/api/stats/users'
        data =   self._me(url)
        sessionModulesStudied = data.get('sessionModulesStudied')
        sessionsCompleted = data.get('sessionsCompleted')
        totalStudyTime = data.get('totalStudyTime')
        KnowledgeScore = totalStudyTime*sessionsCompleted*sessionModulesStudied
        return KnowledgeScore










def Create(SessionId):
    return {
        "platform": "seneca-web",
        "clientVersion": "2.4.128",
        "session": {
            "sessionId": SessionId,
            "sessionType": "adaptive",
            "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
            "sectionIds": ["eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a"],
            "contentIds": [],
            "modulesStudied": 15,
            "modulesTested": 9,
            "modulesCorrect": 9,
            "modulesIncorrect": 0,
            "modulesGaveUp": 0,
            "averageScore": 1,
            "sessionScore": 1,
            "timeStarted": "2022-03-25T19:04:28+00:00",
            "timeFinished": "2022-03-25T23:17:17+00:00",
            "startingProficiency": 0.9662722814208937,
            "endingProficiency": 0.9766666666666667,
            "startingCourseProficiency": 0.012078403517761172,
            "endingCourseProficiency": 0.012208333333333333,
            "endingCourseScore": 0.008849557522123894,
            "completed": True,
            "smartLearningActive": False,
            "signedOut": False,
            "wrongModuleIds": [],
            "totalModulesStudyTime": 0
        },
        "modules": [
            {
                "sessionId": SessionId,
                "moduleOrder": 0,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "321858d0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "321858d1-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "image-description",
                "testingActive": False,
                "content": {
                    "title": "Atomic Model",
                    "words": ["An atom has a small, positively-charged nucleus surrounded by orbiting negatively-charged electrons."],
                    "imageURL": "seneca-image-cdn:///courseImages/physics/5.1.1%20Atomic%20model/5.1.1-Atomic-model-adapted%20(2)-min.png",
                    "image": {
                        "url": "https://image-v2.cdn.app.senecalearning.com/courseImages/physics/5.1.1%20Atomic%20model/5.1.1-Atomic-model-adapted%20(2)-min,f_inside,h_300,w_550.png",
                        "fallbackUrl": None
                    },
                    "renderImage": True
                },
                "userAnswer": {},
                "score": None,
                "moduleScore": {},
                "timeStarted": "2022-03-25T19:04:30+00:00",
                "timeFinished": "2022-03-25T19:10:23+00:00",
                "gaveUp": False,
                "submitted": False,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 1,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "321858d0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32187fe2-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "toggles",
                "testingActive": True,
                "content": {
                    "statement": "Which of the following statements are true of atoms?",
                    "toggles": [
                        {
                            "index1": "They have a positive nucleus",
                            "index0": "They have a negative nucleus",
                            "correctAnswer": 1
                        },
                        {
                            "index0": "Electrons orbit the nucleus",
                            "index1": "Protons orbit the nucleus",
                            "correctAnswer": 0
                        },
                        {
                            "index1": "They are very tiny - a few hundred picometres across",
                            "index0": "They are quite large - about a centimetre across",
                            "correctAnswer": 1
                        }
                    ],
                    "initialAnswer": [0, 0, 0]
                },
                "userAnswer": [1, 0, 1],
                "score": 1,
                "moduleScore": {"score": 1},
                "timeStarted": "2022-03-25T19:10:23+00:00",
                "timeFinished": "2022-03-25T19:10:30+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 2,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "321858d0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32187fe3-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "delve",
                "testingActive": False,
                "content": {
                    "title": "Nucleus",
                    "description": "The nucleus contains [protons](Protons) and neutrons in the centre. [Electrons](Electrons) orbit outside of the nucleus.",
                    "children": [
                        {
                            "title": "Protons",
                            "description": "A small, positively charged particle found in the nucleus.",
                            "parent": "Nucleus"
                        },
                        {
                            "title": "Electrons",
                            "description": "An electron is a small, negatively charged particle that orbits the nucleus.",
                            "parent": "Nucleus"
                        }
                    ]
                },
                "userAnswer": {},
                "score": None,
                "moduleScore": {},
                "timeStarted": "2022-03-25T19:10:30+00:00",
                "timeFinished": "2022-03-25T19:10:31+00:00",
                "gaveUp": False,
                "submitted": False,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 3,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "326feeb0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "326feeb1-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "image-description",
                "testingActive": False,
                "content": {
                    "title": "Structure of an Atom",
                    "words": ["- Atoms are very small and have a radius of about 1 × 10<sup>-10 </sup> metres.\n- The electrons are arranged at different distances from the nucleus (different energy levels). \n- The radius of a nucleus is less than 1/10 000 of the radius of an atom. \n- Most of the mass of an atom is concentrated in the nucleus."],
                    "imageURL": "seneca-image-cdn:///courseImages/physics/AQA%20New%20Modules/6.1.1/structure%20atom.png",
                    "image": {
                        "url": "https://image-v2.cdn.app.senecalearning.com/courseImages/physics/AQA%20New%20Modules/6.1.1/structure%20atom,f_inside,h_300,w_550.png",
                        "fallbackUrl": None
                    },
                    "renderImage": True
                },
                "userAnswer": {},
                "score": None,
                "moduleScore": {},
                "timeStarted": "2022-03-25T19:10:31+00:00",
                "timeFinished": "2022-03-25T19:10:32+00:00",
                "gaveUp": False,
                "submitted": False,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 4,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "326feeb0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "326feeb2-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "concept",
                "testingActive": False,
                "content": {
                    "gifExamples": True,
                    "randomExampleOrder": False,
                    "title": "Size of Atoms vs Nucleus",
                    "explanation": "Atoms have a radius of radius of about 1 × 10<sup>-10 </sup> metres. However, the nucleus of atoms are even smaller. ",
                    "examples": [
                        {
                            "title": "Atoms",
                            "example": "- If an atom was the size of Wembley Stadium, then...",
                            "image": {
                                "url": "https://media.giphy.com/media/l49JNOhwsfiJmdZQs/giphy.mp4",
                                "fallbackUrl": "https://gif.app.senecalearning.com/media/l49JNOhwsfiJmdZQs/giphy.mp4"
                            }
                        },
                        {
                            "title": "Nucleus",
                            "example": "- ...its nucleus would be the size of a garden pea!",
                            "image": {
                                "url": "https://media.giphy.com/media/xTiTnekiQtHA5rcmis/giphy.mp4",
                                "fallbackUrl": "https://gif.app.senecalearning.com/media/xTiTnekiQtHA5rcmis/giphy.mp4"
                            }
                        }
                    ]
                },
                "userAnswer": {},
                "score": None,
                "moduleScore": {},
                "timeStarted": "2022-03-25T19:10:32+00:00",
                "timeFinished": "2022-03-25T19:10:33+00:00",
                "gaveUp": False,
                "submitted": False,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 5,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "326feeb0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "326feeb6-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "toggles",
                "testingActive": True,
                "content": {
                    "statement": "Which of the following has the largest mass?",
                    "toggles": [
                        {
                            "index1": "An atom",
                            "index0": "An atom's nucleus",
                            "correctAnswer": 1
                        },
                        {
                            "index1": "An atom's nucleus",
                            "index0": "An atom's energy levels",
                            "correctAnswer": 1
                        }
                    ],
                    "initialAnswer": [0, 1]
                },
                "userAnswer": [1, 1],
                "score": 1,
                "moduleScore": {"score": 1},
                "timeStarted": "2022-03-25T19:10:33+00:00",
                "timeFinished": "2022-03-25T19:10:35+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 6,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "32ac5b70-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32ac5b71-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "concept",
                "testingActive": False,
                "content": {
                    "randomExampleOrder": False,
                    "title": "The Charges of Sub-Atomic Particles",
                    "explanation": "The 3 different sub-atomic particles (protons, neutrons and electrons) have different relative charges. Atoms have no overall charge (are neutral). In an atom, these charges all cancel each other out. ",
                    "examples": [
                        {
                            "title": "Protons (+1)",
                            "example": "- Protons have a relative charge of +1.\n- They are found in the nucleus.\n- An element’s atomic number is the number of protons it possesses. \n - All atoms of the same element have the same number of protons.",
                            "image": {
                                "url": "https://image-v2.cdn.app.senecalearning.com/courseImages/chemistry/1.1.2%20-%20Model%20of%20the%20atom/1.1.3.1%20Atom,f_cover,h_400,w_600.jpg",
                                "fallbackUrl": None
                            }
                        },
                        {
                            "title": "Electrons (-1)",
                            "example": "- Electrons have a relative charge of -1.\n- They are found in fixed orbits around the nucleus.\n- In any atom, the total number of negative electrons equals the number of positive protons, meaning atoms have no overall electric charge.",
                            "image": {
                                "url": "https://image-v2.cdn.app.senecalearning.com/courseImages/chemistry/1.1.2%20-%20Model%20of%20the%20atom/1.1.3.1%20Atom,f_cover,h_400,w_600.jpg",
                                "fallbackUrl": None
                            }
                        },
                        {
                            "title": "Neutrons (0)",
                            "example": "- Neutrons have a relative charge of 0 - they are neutral.\n- Like protons, they are found in the nucleus.",
                            "image": {
                                "url": "https://image-v2.cdn.app.senecalearning.com/courseImages/chemistry/1.1.2%20-%20Model%20of%20the%20atom/1.1.3.1%20Atom,f_cover,h_400,w_600.jpg",
                                "fallbackUrl": None
                            }
                        }
                    ]
                },
                "userAnswer": {},
                "score": None,
                "moduleScore": {},
                "timeStarted": "2022-03-25T19:10:35+00:00",
                "timeFinished": "2022-03-25T19:10:36+00:00",
                "gaveUp": False,
                "submitted": False,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 7,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "32ac5b70-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32ac5b78-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "multiple-choice",
                "testingActive": True,
                "content": {
                    "question": "What is the overall charge of the nucleus of an atom?",
                    "answers": ["Negative", "Positive", "Neutral"],
                    "correctAnswerIndex": 1
                },
                "userAnswer": {"index": 1},
                "score": 1,
                "moduleScore": {"score": 1},
                "timeStarted": "2022-03-25T19:10:36+00:00",
                "timeFinished": "2022-03-25T19:10:39+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 8,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "32ac5b70-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32ac5b72-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "delve",
                "testingActive": False,
                "content": {
                    "title": "Atoms",
                    "description": "Atoms are made up of a [nucleus](Nucleus) and orbiting [electrons](Electrons).",
                    "children": [
                        {
                            "description": "A nucleus is at the centre of an atom. It contains [protons](Protons) and [neutrons](Neutrons). ",
                            "parent": "Atoms",
                            "title": "Nucleus",
                            "children": [
                                {
                                    "title": "Protons",
                                    "description": "The positive charge of protons cancels out the negative charge of the electrons. ",
                                    "parent": "Nucleus"
                                },
                                {
                                    "title": "Neutrons",
                                    "description": "Neutrons have the same mass as protons but have no charge. ",
                                    "parent": "Nucleus"
                                }
                            ]
                        },
                        {
                            "title": "Electrons",
                            "description": "Electrons are negatively charged particles that orbit the nucleus at fixed distances. ",
                            "parent": "Atoms"
                        }
                    ]
                },
                "userAnswer": {},
                "score": None,
                "moduleScore": {},
                "timeStarted": "2022-03-25T19:10:39+00:00",
                "timeFinished": "2022-03-25T19:10:40+00:00",
                "gaveUp": False,
                "submitted": False,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 9,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "321858d0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32187fe1-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "wordfill",
                "testingActive": True,
                "content": {
                    "words": [
                        "An atom has a small, ",
                        {
                            "otherPermittedWords": ["positive", "+1"],
                            "word": "positively",
                            "testing": True
                        },
                        " charged nucleus surrounded by orbiting negatively charged ",
                        {
                            "otherPermittedWords": [],
                            "word": "electrons",
                            "testing": True
                        },
                        "."
                    ],
                    "imageURL": "https://image-v2.cdn.app.senecalearning.com/courseImages/physics/5.1.1.1%20-%20Model%20of%20atom-min,f_cover,h_250,w_1024.png",
                    "renderImage": False
                },
                "userAnswer": {"words": {
                        "positively": "positively",
                        "electrons": "electrons"
                    }},
                "score": 1,
                "moduleScore": {
                    "words": {
                        "positively": True,
                        "electrons": True
                    },
                    "score": 1
                },
                "timeStarted": "2022-03-25T19:10:40+00:00",
                "timeFinished": "2022-03-25T19:10:43+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 10,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "326feeb0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "326feeb3-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "multiple-choice",
                "testingActive": True,
                "content": {
                    "question": "In what environment would the relative mass of an atom change?",
                    "answers": ["On the Moon", "Mass is constant across all environments", "In deep space"],
                    "correctAnswerIndex": 1,
                    "image": {
                        "url": "https://image-v2.cdn.app.senecalearning.com/courseImages/physics/AQA%20New%20Modules/6.1.1/structure%20atom,f_inside,h_267,w_400.png",
                        "fallbackUrl": None
                    }
                },
                "userAnswer": {"index": 1},
                "score": 1,
                "moduleScore": {"score": 1},
                "timeStarted": "2022-03-25T19:10:43+00:00",
                "timeFinished": "2022-03-25T19:10:49+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 11,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "32ac5b70-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32ac5b77-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "grid",
                "testingActive": True,
                "content": {
                    "title": "Relative charge of sub-atomic particles",
                    "definitions": [
                        {
                            "word": "Neutron",
                            "text": "0",
                            "testing": True
                        },
                        {
                            "word": "Electron",
                            "text": "-1",
                            "testing": True
                        },
                        {
                            "word": "Proton",
                            "text": "+1",
                            "testing": True
                        }
                    ]
                },
                "userAnswer": {"definitions": {
                        "Neutron": "Neutron",
                        "Electron": "Electron",
                        "Proton": "Proton"
                    }},
                "score": 1,
                "moduleScore": {
                    "definitions": {
                        "Neutron": True,
                        "Electron": True,
                        "Proton": True
                    },
                    "score": 1
                },
                "timeStarted": "2022-03-25T19:10:49+00:00",
                "timeFinished": "2022-03-25T19:10:51+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 12,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "321858d0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32187fe8-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "multiple-choice",
                "testingActive": True,
                "content": {
                    "question": "What do we call a small, negatively charged particle that orbits an atom's nucleus?",
                    "answers": ["Proton", "Element", "Ion", "Electron", "Compound"],
                    "correctAnswerIndex": 3
                },
                "userAnswer": {"index": 3},
                "score": 1,
                "moduleScore": {"score": 1},
                "timeStarted": "2022-03-25T19:10:51+00:00",
                "timeFinished": "2022-03-25T19:10:56+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 13,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "326feeb0-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "326feeb7-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "wordfill",
                "testingActive": True,
                "content": {
                    "words": [
                        "An atom's ",
                        {
                            "otherPermittedWords": [],
                            "word": "electrons",
                            "testing": True
                        },
                        " are arranged at different distances from the nucleus (different energy levels). ."
                    ],
                    "fact": True,
                    "renderImage": False
                },
                "userAnswer": {"words": {"electrons": "electrons"}},
                "score": 1,
                "moduleScore": {
                    "words": {"electrons": True},
                    "score": 1
                },
                "timeStarted": "2022-03-25T19:10:56+00:00",
                "timeFinished": "2022-03-25T19:11:04+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            },
            {
                "sessionId": SessionId,
                "moduleOrder": 14,
                "courseId": "2670ac10-1d69-11e8-bf76-f14a3ef7c0e6",
                "sectionId": "eb52d2e0-1d6b-11e8-8e43-0b8b5e91224a",
                "contentId": "32ac5b70-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleId": "32ac5b73-1d6c-11e8-8e43-0b8b5e91224a",
                "moduleType": "list",
                "testingActive": True,
                "content": {
                    "pretest": "SOMETIMES",
                    "multipleChoice": {
                        "question": "How many types of sub-atomic particles are there in an atom?",
                        "answers": ["5", "2", "4", "3"],
                        "correctAnswerIndex": 3
                    },
                    "list": {
                        "statement": "Sub-atomic particles",
                        "values": [
                            {
                                "value": [
                                    {
                                        "otherPermittedWords": [],
                                        "word": "proton",
                                        "caps": "Proton"
                                    }
                                ],
                                "testing": True
                            },
                            {
                                "value": [
                                    {
                                        "otherPermittedWords": [],
                                        "word": "neutron",
                                        "caps": "Neutron"
                                    }
                                ],
                                "testing": True
                            },
                            {
                                "value": [
                                    {
                                        "otherPermittedWords": [],
                                        "word": "electron",
                                        "caps": "Electron"
                                    }
                                ],
                                "testing": True
                            }
                        ]
                    }
                },
                "userAnswer": {
                    "multipleChoice": {"index": 3},
                    "list": ["Proton", "Electron", "Neutron"]
                },
                "score": 1,
                "moduleScore": {
                    "score": 1,
                    "multipleChoice": {"score": 1},
                    "list": {
                        "score": 1,
                        "hits": [True, True, True]
                    }
                },
                "timeStarted": "2022-03-25T19:11:04+00:00",
                "timeFinished": "2022-03-25T19:11:17+00:00",
                "gaveUp": False,
                "submitted": True,
                "completed": True
            }
        ]
    }


from setuptools import setup, find_packages
import codecs
import os



VERSION = '2.0.0b9'
DESCRIPTION = 'senecalearning.com API wrapper'

# Setting up
setup(
    name="SenecaPy",
    version=VERSION,
    author="1azza (Larry Rennoldson)",
    author_email="rennoldsonlarry@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['requests', 'websocket-client'],
    keywords=['python', 'cheat', 'seneca', 'API', 'wrapper', 'requests', 'senecalearning', 'senecalearning.com', 'homework'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



from pprint import pprint
import requests
from seneca.Modules.key import  APIKEY
import json
from seneca.Modules.errors import (ServerError, InvalidCredentials)
import logging
class Token:
    '''
    This is a class for creating a token object.
    :param email: The email of the user.
    :param password: The password of the user.
    :return: A token object.
    '''
    def __init__(self,  email:str, password:str):
        url = "https://www.googleapis.com/identitytoolkit/v3/relyingparty/verifyPassword"

        querystring = {"key": APIKEY}

        payload = {
            "email": email,
            "password": password,
            "returnSecureToken": True
        }
        self.headers = {
            "authority": "www.googleapis.com",
            "pragma": "no-cache",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36",
        }
        logging.info(f'Logging in as: {email}')
        response = requests.request("POST", url, json=payload, headers=self.headers, params=querystring)
        self.refereshToken = response.json().get('refreshToken')
        if response.status_code == 400:
            raise InvalidCredentials()
        elif response.status_code != 200:
            raise ServerError()
        
    def Refresh(self):
        '''
        Refreshes the idToken.
        :return: The idToken.
        '''
        url = "https://securetoken.googleapis.com/v1/token"
        querystring = {"key": APIKEY}
        payload = f"grant_type=refresh_token&refresh_token={self.refereshToken}"
        headers = {
            "content-type": "application/x-www-form-urlencoded",
        }
        response = requests.request(
            "POST", url, data=payload, headers=headers, params=querystring)
        if response.status_code == 200:
            logging.info('Refreshed idToken')
        elif response.status_code == 400:
            raise InvalidCredentials()
        else:
            raise ServerError()
            
        self.idToken = response.json().get('id_token')
        self.user_id = response.json().get('user_id')
        
        if self.idToken == None:
            logging.critical('No idToken found')
        else:
            self.userKeys = json.dumps(response.json(),   indent=4, sort_keys=True)
            return self.idToken
        


from .Modules.memories import Memories
from .Modules.token import Token
from .Modules.info import Info
from .Modules.course import Course
from .Modules.sessions import Sessions
from .Modules.profile import Profile
import logging
class User():
    '''
    A class that defines the user.
    :param username: The username of the user.
    :param password: The password of the user.
    :return: A user object.
    '''
    def __init__(self, username:str, password:str):
        token = Token(username, password)
        self.idToken = token.Refresh()
        self.info = Info(self.idToken)
        self.Username = self.info.getAccountInfo().get('displayName')
        logging.info(f'Logged in as {self.Username}')
        self.Tokens = token.userKeys
        self.id = token.user_id
        self.profile = Profile(self.idToken)
        self.memories = Memories(self.idToken)
        self.Session = Sessions
        self.profile = Profile(self.idToken)        
course = Course()